// Include files
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// Defines
#define MYAPP "openFile "
#define MYFILE "Filetest"

// Main function
// NOTE: close() and system() return code are not checked just to avoid
//  too much cluttering of the code, but they SHOULD BE CHECKED.
int main(int argc, char **argv)
{
  int fd;
  int n, nw;
  char buf[1024];
  
  // delete file to create the precondition of this test procedure
  // unlink() returns error also if the input file does not exist,
  //  but for us it is fine, so error is not checked!
  unlink(MYFILE);

  printf("\n");
  
  //-----------------------
  printf("%s: opening a NON existing file \"%s\" using O_RDWR ...\n",
	 MYAPP,MYFILE);
  if ((fd=open(MYFILE,O_RDWR)) == -1) {
    perror(MYAPP "ERROR");
  }
  else {
    printf("%s: file has been opened with fd=%d\n",MYAPP,fd);
    printf("%s: now closing fd=%d\n",MYAPP,fd);
    close(fd);
  }
  
  printf("[Enter to continue]\n");
  getchar();
  
  //------------------------
  printf("%s: opening a NON existing file \"%s\" using O_RDWR | O_CREAT and 0660 mode ...\n",
	 MYAPP,MYFILE);
  if ((fd=open(MYFILE,O_RDWR | O_CREAT,0660)) == -1) {
    perror(MYAPP "ERROR");
  }
  else {
    printf("%s: file has been opened with fd=%d\n",MYAPP,fd);
    printf("%s: now closing fd=%d\n",MYAPP,fd);
    close(fd);
  }

  // Write some stuff to the file
  printf("%s: Write some stuff to the file (using \"echo SOME STUFF > %s\")\n",MYAPP, MYFILE);
  system("echo SOME STUFF > " MYFILE);

  // Display info about the file
  printf("%s: File information and content:\n",MYAPP);
  printf("-------------------------------------\n");
  system("ls -l " MYFILE);
  printf("-------------------------------------\n");
  system("cat " MYFILE);
  printf("-------------------------------------\n");

  printf("[Enter to continue]\n");
  getchar();
  
  //----------------------- 
  printf("%s: opening an EXISTING file \"%s\" using O_RDWR ...\n",
	     MYAPP,MYFILE);
  if ((fd=open(MYFILE,O_RDWR)) == -1) {
    perror(MYAPP "ERROR");
  }
  else {
    printf("%s: file has been opened with fd=%d\n",MYAPP,fd);
    printf("%s: now closing fd=%d\n",MYAPP,fd);
    close(fd);
  }
  
  printf("[Enter to continue]\n");
  getchar();
  
  //-----------------------
  printf("%s: opening an existing file \"%s\" using O_RDRW | O_CREAT and 0660 mode ...\n",
	 MYAPP,MYFILE);
  if ((fd=open(MYFILE,O_RDWR | O_CREAT,0660)) == -1) {
    perror(MYAPP "ERROR");
  }
  else {
    printf("%s: file has been opened with fd=%d\n",MYAPP,fd);
    printf("%s: now closing fd=%d\n",MYAPP,fd);
    close(fd);
  }
  
  printf("[Enter to continue]\n");
  getchar();
  
  //-----------------------
  printf("%s: opening an existing file \"%s\" using O_RDWR | O_CREAT | O_EXCL and 0660 mode ...\n",
	 MYAPP,MYFILE);
  if ((fd=open(MYFILE,O_RDWR | O_CREAT | O_EXCL,0660)) == -1) {
    perror(MYAPP "ERROR");
  }
  else {
    printf("%s: file has been opened with fd=%d\n",MYAPP,fd);
    printf("%s: now closing fd=%d\n",MYAPP,fd);
    close(fd);
  }
  
  printf("[Enter to continue]\n");
  getchar();
  
  //-----------------------   
  printf("%s: opening an existing file \"%s\" using O_RDWR | O_EXCL ...\n",
	 MYAPP,MYFILE);
  if ((fd=open(MYFILE,O_RDWR | O_EXCL)) == -1) {
    perror(MYAPP "ERROR");
  }
  else {
    printf("%s: file has been opened with fd=%d\n",MYAPP,fd);
    
    // write some stuff 
    strcpy(buf,"Some stuff just to write some bytes\n");
    printf("%s: write some bytes into file\n",MYAPP);
    n=strlen(buf);
    if ((nw=write(fd,buf,n)) != n)
      printf(MYAPP "ERROR: only %d bytes written to fd=%d (instead of %d bytes)\n",
	     nw, fd, n);
    
    printf("%s: now closing fd=%d\n",MYAPP,fd);
    close(fd);
  }
  
  printf("\n");
  
  // Display info about the file
  printf("%s: File information and content:\n",MYAPP);
  printf("-------------------------------------\n");
  system("ls -l " MYFILE);
  printf("-------------------------------------\n");
  system("cat " MYFILE);
  printf("-------------------------------------\n");
  
  printf("[Enter to continue]\n");
  getchar();
  
  //----------------------
  printf("%s: opening an existing file \"%s\" using O_RDWR | O_APPEND ...\n",
	 MYAPP,MYFILE);
  if ((fd=open(MYFILE,O_RDWR | O_APPEND)) == -1) {
    perror(MYAPP "ERROR");
  }
  else {
    printf("%s: file has been opened with fd=%d\n",MYAPP,fd);
    
    // write some additional stuff 
    strcpy(buf,"This is a line that should be appended to the file\n");
    printf("%s: write some bytes into file (appended)\n",MYAPP);
    n=strlen(buf);
    if ((nw=write(fd,buf,n)) != n)
      printf(MYAPP "ERROR: only %d bytes written to fd=%d (instead of %d bytes)\n",
	     nw, fd, n);
    
    printf("%s: now closing fd=%d\n",MYAPP,fd);
    close(fd);
  }
  
  printf("\n");
  
  // Display info about the file
  printf("%s: File information and content:\n",MYAPP);
  printf("-------------------------------------\n");
  system("ls -l " MYFILE);
  printf("-------------------------------------\n");
  system("cat " MYFILE);
  printf("-------------------------------------\n");
  
  printf("[Enter to continue]\n");
  getchar();
  
  //----------------------
  printf("%s: opening an existing file \"%s\" using O_RDWR | O_TRUNC ...\n",
	 MYAPP,MYFILE);
  if ((fd=open(MYFILE,O_RDWR | O_TRUNC)) == -1) {
    perror(MYAPP "ERROR");
  }
  else {
    // write some new stuff 
    printf("%s: file has been opened with fd=%d\n",MYAPP,fd);
    
    strcpy(buf,"Now the file should contain only this line\n");
    printf("%s: write some bytes into file (the file should be empty)\n",MYAPP);
    n=strlen(buf);
    if ((nw=write(fd,buf,n)) != n)
      printf(MYAPP "ERROR: only %d bytes written to fd=%d (instead of %d bytes)\n",
	     nw, fd, n);

    printf("%s: now closing fd=%d\n",MYAPP,fd);
    close(fd);
  }
  
  printf("\n");
  
  // Display info about the file
  printf("%s: File information and content:\n",MYAPP);
  printf("-------------------------------------\n");
  system("ls -l " MYFILE);
  printf("-------------------------------------\n");
  system("cat " MYFILE);
  printf("-------------------------------------\n");
  
  printf("[Enter to continue]\n");
  getchar();
  
  //----------------------
  printf("%s: opening an existing file \"%s\" using O_RDONLY | O_TRUNC ...\n",
	 MYAPP,MYFILE);
  if ((fd=open(MYFILE,O_RDONLY | O_TRUNC)) == -1) {
    perror(MYAPP "ERROR");
  }
  else {
    printf("%s: now closing fd=%d\n",MYAPP,fd);
    close(fd);
  }
  
  printf("\n");

  // Display info about the file
  printf("%s: File information and content:\n",MYAPP);
  printf("-------------------------------------\n");
  system("ls -l " MYFILE);
  printf("-------------------------------------\n");
  system("cat " MYFILE);
  printf("-------------------------------------\n");
 
  printf("\n");

  //----------------------
  printf("%s: opening an existing file \"%s\" using O_RDONLY | O_APPEND ...\n",
	 MYAPP,MYFILE);
  if ((fd=open(MYFILE,O_RDONLY | O_APPEND)) == -1) {
    perror(MYAPP "ERROR");
  }
  else {
    // write some new stuff 
    printf("%s: file has been opened with fd=%d\n",MYAPP,fd);
    
    strcpy(buf,"Now the file should contain only this line\n");
    printf("%s: write some bytes into file (the file should be empty)\n",MYAPP);
    n=strlen(buf);
    if ((nw=write(fd,buf,n)) != n)
      printf(MYAPP "ERROR: only %d bytes written to fd=%d (instead of %d bytes)\n",
	     nw, fd, n);

    printf("%s: now closing fd=%d\n",MYAPP,fd);
    close(fd);
  }
  
  printf("\n");
  
  // Display info about the file
  printf("%s: File information and content:\n",MYAPP);
  printf("-------------------------------------\n");
  system("ls -l " MYFILE);
  printf("-------------------------------------\n");
  system("cat " MYFILE);
  printf("-------------------------------------\n");
  
  printf("[Enter to continue]\n");
  getchar();

  exit(EXIT_SUCCESS);
}
